#!/usr/bin/python3
"""Delete old messages that are not to be archived."""
from datetime import datetime, timedelta
from typing import Sequence, cast

import click
from pytz import UTC

from messenger.commands.utils import Verbosity, echo, messenger_command, method_option
from messenger.constants import MessageStatus, TransportMethod
from messenger.database import atomic
from messenger.models import Record
from messenger.settings import SETTINGS


@messenger_command()
@method_option
@click.option('--days', type=int, help="Set custom number of days before message deletion.")
def main(methods: Sequence[TransportMethod], days: int):
    """Delete old messages that are not to be archived."""
    if days is not None:
        SETTINGS['clean_days'] = days
    for method in methods:
        with atomic() as session:
            # mypy gets confused by lru_cache, see https://github.com/python/mypy/issues/5858
            model_cls = cast(Record, method.model_cls)
            cutoff = datetime.now(UTC) - timedelta(days=SETTINGS['clean_days'])
            query = session.query(method.model_cls).filter(model_cls.create_datetime <= cutoff)
            # Delete error messages
            deleted = query.filter_by(status=MessageStatus.ERROR).delete()
            # Delete messages that are not to be archived
            deleted += query.filter_by(archive=False).filter(model_cls.status != MessageStatus.PENDING).delete()
        echo('Deleted {} {}s.'.format(deleted, method), Verbosity.INFO)


if __name__ == '__main__':
    main()
